import 'package:flutter/material.dart';

/// Tipo de animaciones permitidas
enum AnimationType { normal, fadein }

/// Nombre de la clase
class RouteTransition {
  /// Main class, [context] es el BuildContext de la aplicacion en ese momento
  ///[child] es el widget a navegar, [animation] es el tipo de animacion deseada ,
  ///[duration] es el tiempo que deseamos que dure la animacion,[replacement] permite
  ///que la pagina anterior se borre del stack.
  final BuildContext context;
  final Widget child;
  final AnimationType animation;
  final Duration duration;
  final bool replacement;

// se inicializan las variables
  RouteTransition({
    this.replacement = false,
    @required this.context,
    @required this.child,
    this.animation = AnimationType.normal,
    this.duration = const Duration(milliseconds: 300),
  }) {
    // condiciones para las animaciones
    switch (this.animation) {
      case AnimationType.normal:
        _normalTransition();
        break;
      case AnimationType.fadein:
        _fadeinTransition();
        break;
      default:
    }
  }
  // handler animacion normal
  void _normalTransition() =>
      _replacement(MaterialPageRoute(builder: (_) => this.child));

  // handler animacion fadein
  void _fadeinTransition() {
    final route = PageRouteBuilder(
      pageBuilder: (_, __, ___) => this.child,
      transitionDuration: this.duration,
      transitionsBuilder: (_, animation, __, child) {
        return FadeTransition(
          child: child,
          opacity: Tween<double>(begin: 0.0, end: 1.0).animate(
            CurvedAnimation(parent: animation, curve: Curves.easeInOut),
          ),
        );
      },
    );
    _replacement(route);
  }

// metodo _replacement
  void _replacement(Route route) {
    if (this.replacement) {
      Navigator.pushReplacement(this.context, route);
    } else {
      Navigator.push(this.context, route);
    }
  }
}
